'use strict';

var _require = require('babel-plugin-macros'),
    createMacro = _require.createMacro;

var _require2 = require('./visitor'),
    visitor = _require2.visitor;

module.exports = createMacro(myMacro);

function myMacro(_ref) {
  var references = _ref.references,
      _ = _ref._,
      babel = _ref.babel;
  var _references$macro = references.macro,
      macro = _references$macro === undefined ? [] : _references$macro,
      _references$default = references.default,
      defaultImport = _references$default === undefined ? [] : _references$default;
  var t = babel.types;

  macro.forEach(function (referencePath) {
    if (isValidMacroInvocation(referencePath)) {
      var prog = referencePath.find(function (x) {
        return x.isProgram();
      });
      prog.traverse(visitor(t));
      // import React if not already imported
      if (!isReactAlreadyImported(prog, defaultImport)) {
        var importSpecifiers = [t.importDefaultSpecifier(t.identifier('React'))];
        var source = t.stringLiteral('react');
        var ast = t.importDeclaration(importSpecifiers, source);
        prog.unshiftContainer('body', ast);
      }
      // remove the macro() reference as last step
      var par = referencePath.findParent(function (x) {
        return x.isExpressionStatement();
      });
      par.remove();
    }
  });
}

function isReactAlreadyImported(prog, defaultImport) {
  if (prog.scope.references.React && defaultImport.length < 1) return true;
  if (defaultImport.length < 1) return false;
  if (defaultImport[0].node.name === 'React') return false;
  return true;
}

function isValidMacroInvocation(path) {
  // strictly only one way to call this thing
  var par = path.parentPath;
  if (!par.isCallExpression()) return false;
  par = par.parentPath;
  if (!par.isExpressionStatement()) return false;
  par = par.parentPath;
  if (!par.isProgram()) return false;
  return true;
}